// C Source File
// Created 6/13/01; 9:35:13 AM
/*
RLE encoding/decoding by Gregory Dietsche 2001.

	Description:
		Uses Run Length Encoding to compress data. This is different from the
		standard rle methods normally associated with TI-calculators. It uses 
		a two byte encoding method for runs and can provide better compression in
		some situations(usually not though) and is faster.
		It should work well on data with most (read: all) byte values below 0x80. If a byte
		is 0x80 or larger, it will take up 2 bytes instead of one.
		
	Input:
		out	pointer to buffer for output
		in	pointer to input stream
		len	number of bytes contained in stream in
		
	Output:
		len	the number of bytes in buffer out. If out was NULL then len will be
		the encoded size of buffer in; no decoding will be peformed.
		
	NOTES:
		Sometimes the encoded size of the data will be larger than then original
		data.

*/
#define RunMarker 0x80
#define FreqMask 0x7f		//RunMarker - 1

unsigned long rle3_encode(register unsigned char *out, register unsigned char *in, register unsigned long len)
{
	register unsigned long posIn=0;
	register unsigned long posOut=0;
	register unsigned char byte;
	register unsigned char freq;
	
	while(posIn<len)
	{
		byte=in[posIn];
		freq=1;
		
		while(in[++posIn]==byte && freq<FreqMask && posIn<len)
			freq++;
		
		if(freq==1 && byte < RunMarker)
		{
				if(out)
					out[posOut++]=byte;
				else
					posOut++;
		}	
		else
		{
			if(out)
			{
				out[posOut++]=RunMarker | freq;
				out[posOut++]=byte;
			}
			else
				posOut+=2;
		}
	}
	
	return posOut;
}

