// C Source File
// Created 6/12/01; 5:34:49 PM
/*
RLE encoding/decoding by Gregory Dietsche 2001.

	Description:
		Uses the standard TI Run Length Encoding system to compress data.
		
	Input:
		out	pointer to buffer for output
		in	pointer to input stream
		len	number of bytes contained in stream in
		
	Output:
		len	the number of bytes in buffer out. If out was NULL then len will be
		the encoded size of buffer in; no decoding will be peformed.
		
	NOTES:
		Sometimes the encoded size of the data will be larger than then original
		data.

*/

#define RunMarker 0x91

unsigned long rle_encode(register unsigned char *out, register unsigned char *in, register unsigned long len)
{
	register unsigned long posIn=0;
	register unsigned long posOut=0;
	register unsigned char freq;
	register unsigned char byte;
	register unsigned char i;
	
	while(posIn<len)
	{
		byte=in[posIn];
		freq=0;

		while(in[++posIn]==byte && freq!=255 && posIn<len)
			freq++;
		
		if(freq<3 && byte!=RunMarker)
		{
			freq++;
		
			if(out)
				for(i=0;i<freq;i++)
					out[posOut++]=byte;
			else
				posOut+=freq;
		}	
		else
		{
			freq++;
		
			if(out)
			{	
				out[posOut++]=RunMarker;
				out[posOut++]=byte;
				out[posOut++]=freq;
			}
			else
				posOut+=3;
		}
	}
	
	return posOut;
}
