// C Source File
// Created 6/13/01; 11:39:16 PM
/*
RLE encoding/decoding by Gregory Dietsche 2001.

	Description:
		encodes or decodes a file (it is smart and can figure out what to do)
		
	Input:
		out	pointer to a standard c string containing the name of the file to write to.
				If the file exists, it will be overwritten.
		in	pointer to a standard c string containing the name of the file to read from
		
	Output:
		RLE_ERROR, RLE_ENCODED, RLE_DECODED
		
	NOTES:
		Sometimes the encoded size of the data will be larger than then original
		data. You can't use this routine because I need to copy some source from tigcclib.a
		to here for it to work in the archive.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "RLE.h"

int rle_file(char *out, char *in)
{	
	FILE *fin, *fout;
	char *ptr1, *ptr2;
	short inlen;
	unsigned char tmp[6];
	static unsigned char rle_tag[]={0,'R','L','E',0,0xF8};
	
	fin=fopen(in,"rb");
	fout=fopen(out,"wb");
	
	if(fin==NULL || fout==NULL)
		return RLE_ERROR;
	
	fseek(fin,-6,SEEK_END);
	fread(tmp,1,6,fin);
	
	//get the file length
	//fseek(fin,0,SEEK_END);
	inlen=ftell(fin);
	fseek(fin,0,SEEK_SET);
	
	ptr1=malloc(inlen);
	fread(ptr1,1,inlen,fin);
	fclose(fin);

	if(memcmp(tmp,rle_tag,sizeof(rle_tag)))
	{
			ptr2=malloc(rle_encode(NULL,ptr1,inlen));
	
			inlen=rle_encode(ptr2,ptr1,inlen);
			free(ptr1);

			fwrite(ptr2,1,inlen,fout);
			fwrite(rle_tag,1,sizeof(rle_tag),fout);
	
			free(ptr2);
			fclose(fout);
	
			return RLE_ENCODED;
		}
		else
		{
			inlen-=6;
			ptr2=malloc(rle_decode(NULL,ptr1,inlen));
	
			inlen=rle_decode(ptr2,ptr1,inlen);
			free(ptr1);

			fwrite(ptr2,1,inlen,fout);
	
			free(ptr2);
			fclose(fout);
	
			return RLE_DECODED;
		}
}
